<?php

namespace Drupal\entity_legal;

use Drupal\views\ViewEntityInterface;

/**
 * Provides a BC layer views providing old configurations.
 *
 * This is heavily inspired from \Drupal\views\ViewsConfigUpdater
 *
 * @see \Drupal\views\ViewsConfigUpdater
 * @see \views_view_presave()
 */
class EntityLegalViewsConfigUpdater {

  /**
   * Performs all required updates.
   *
   * @param \Drupal\views\ViewEntityInterface $view
   *   The View to update.
   *
   * @return bool
   *   Whether the view was updated.
   */
  public function updateAll(ViewEntityInterface $view): bool {
    return $this->processDisplayHandlers($view, FALSE, function (&$handler, $handler_type, $key, $display_id) use ($view) {
      $changed = FALSE;
      if ($this->processString2SerialUpdateHandler($handler, $handler_type)) {
        $changed = TRUE;
      }
      return $changed;
    });
  }

  /**
   * Processes all display handlers.
   *
   * @param \Drupal\views\ViewEntityInterface $view
   *   The View to update.
   * @param bool $return_on_changed
   *   Whether processing should stop after a change is detected.
   * @param callable $handler_processor
   *   A callback performing the actual update.
   *
   * @return bool
   *   Whether the view was updated.
   */
  protected function processDisplayHandlers(ViewEntityInterface $view, $return_on_changed, callable $handler_processor): bool {
    $changed = FALSE;
    $displays = $view->get('display');
    $handler_types = ['field', 'argument', 'sort', 'relationship', 'filter'];

    foreach ($displays as $display_id => &$display) {
      foreach ($handler_types as $handler_type) {
        $handler_type_plural = $handler_type . 's';
        if (!empty($display['display_options'][$handler_type_plural])) {
          foreach ($display['display_options'][$handler_type_plural] as $key => &$handler) {
            if ($handler_processor($handler, $handler_type, $key, $display_id)) {
              $changed = TRUE;
              if ($return_on_changed) {
                return $changed;
              }
            }
          }
        }
      }
    }

    if ($changed) {
      $view->set('display', $displays);
    }

    return $changed;
  }

  /**
   * Updates the timestamp fields settings by adding time diff and tooltip.
   *
   * @param \Drupal\views\ViewEntityInterface $view
   *   The View to update.
   *
   * @return bool
   *   Whether the view was updated.
   */
  public function needsString2SerialUpdate(ViewEntityInterface $view): bool {
    return $this->processDisplayHandlers($view, FALSE, function (array &$handler, string $handler_type): bool {
      return $this->processString2SerialUpdateHandler($handler, $handler_type);
    });
  }

  /**
   * Processes timestamp fields settings by adding time diff and tooltip.
   *
   * @param array $handler
   *   A display handler.
   * @param string $handler_type
   *   The handler type.
   *
   * @return bool
   *   Whether the handler was updated.
   */
  protected function processString2SerialUpdateHandler(array &$handler, string $handler_type): bool {
    $entity_types = [
      'entity_legal_document_version' => 'name',
      'entity_legal_document_acceptance' => 'document_version_name',
    ];
    if (
      isset($handler['entity_type'])
      && isset($entity_types[$handler['entity_type']])
      && isset($handler['entity_field'])
      && (
        $handler['entity_field'] === $entity_types[$handler['entity_type']]
        || $handler['entity_field'] === 'vid'
      )
    ) {
      $handler['entity_field'] = 'vid';
      $handler['field'] = 'vid';
      return TRUE;
    }
    return FALSE;
  }

}
